// -*- C++ -*-
/*!
 * @file  RunControl.cpp
 * @brief RunControl component
 * $Date$ 2008/11/3
 * @author Nemoto,Mizoguchi lab,Tokyo University of Science
 * @mail:j7507640@ed.noda.tus.ac.jp
 * $Id$
 */

#include "RunControl.h"
#include <iostream>
using namespace std;

// Module specification
// <rtc-template block="module_spec">
static const char* runcontrol_spec[] =
  {
    "implementation_id", "RunControl",
    "type_name",         "RunControl",
    "description",       "RunControl component",
    "version",           "1.0",
    "vendor",            "Nemoto,Mizoguchi Lab,Tokyo University of Science",
    "category",          "RobotModule",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "10",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.max_mps", "1.0",
    "conf.default.max_rps", "1.5",
    "conf.default.min_mps", "0.0",
    "conf.default.min_rps", "-1.5",
    "conf.default.accelerate_mps", "0.5",
    "conf.default.accelerate_rps", "0.5",

    ""
  };
// </rtc-template>

RunControl::RunControl(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_TargetPosIn("TargetPos", m_TargetPos),
    m_OdometryIn("Odometry", m_Odometry),
    m_VelocityOut("Velocity", m_Velocity),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("TargetPos", m_TargetPosIn);
  registerInPort("Odometry", m_OdometryIn);
  
  // Set OutPort buffer
  registerOutPort("Velocity", m_VelocityOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

RunControl::~RunControl()
{
}


RTC::ReturnCode_t RunControl::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("max_mps", m_max_mps, "1.0");
  bindParameter("max_rps", m_max_rps, "1.5");
  bindParameter("min_mps", m_min_mps, "0.0");
  bindParameter("min_rps", m_min_rps, "-1.5");
  bindParameter("accelerate_mps", m_accelerate_mps, "0.5");
  bindParameter("accelerate_rps", m_accelerate_rps, "0.5");
  
  // </rtc-template>
  return RTC::RTC_OK;
}



/*
RTC::ReturnCode_t RunControl::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RunControl::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RunControl::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t RunControl::onActivated(RTC::UniqueId ec_id)
{

    robot_x=0,robot_y=0;
      
    speed=0,turn=0;
    goal_x=0,goal_y=0;
    now_t=0,old_t=0;

    old_speed=0;
    old_turn=0;

    m_Velocity.v=speed;
    m_Velocity.w=turn;
    m_VelocityOut.write();
    
    return RTC::RTC_OK;
}



RTC::ReturnCode_t RunControl::onDeactivated(RTC::UniqueId ec_id)
{
    speed=0,turn=0;
    m_Velocity.v=speed;
    m_Velocity.w=turn;
    m_VelocityOut.write();

    return RTC::RTC_OK;
}



RTC::ReturnCode_t RunControl::onExecute(RTC::UniqueId ec_id)
{
    static double robot_theta;
    usleep(10*1000);
    
    if (m_OdometryIn.isNew()) {
	m_OdometryIn.read();
	robot_x=m_Odometry.x;
	robot_y=m_Odometry.y;
	robot_theta=m_Odometry.theta;
    }
    
    
/*    if (m_TargetPosIn.isNew()) {		//inportからゴールの位置を取得してくる。
	m_TargetPosIn.read();
	goal_x=m_TargetPos.x;
	goal_y=m_TargetPos.y;
    }
*/
    
static double goal_x = 5.0;		//inportからゴールの位置を取得してくるのではなくゴールの位置を指定したい場合。
static double goal_y = 5.0;		//inportからゴールの位置を取得してくるのではなくゴールの位置を指定したい場合。


/*    if((goal_x!=0)&&(goal_y!=0)){
       	follow_velocity(goal_x,goal_y,robot_x,robot_y,speed,turn);
    }
*/

    if((goal_x - robot_x > 1.0)&&(goal_y - robot_y > 1.0)){	//ゴールの位置とロボットの位置が１[m]より大きければ速度計算を行う。
       	follow_velocity(goal_x,goal_y,robot_x,robot_y,speed,turn);
    }
    
    else{
	speed=0.0;
	turn=0.0;	
    }

    
    if(speed>m_max_mps) speed=m_max_mps;
    if(speed<m_min_mps) speed=m_min_mps;
    if(turn>m_max_rps) turn=m_max_rps;
    if(turn<m_min_rps) turn=m_min_rps;

    now_t=timer_ms();
    if((now_t!=0)&&(old_t!=0)&&((now_t-old_t)>0)){

	mps_variation=(now_t-old_t)/1000.0*m_accelerate_mps;
	rps_variation=(now_t-old_t)/1000.0*m_accelerate_rps;

	if(fabs(speed-old_speed)>mps_variation){
	    speed=old_speed+(speed-old_speed)/fabs(speed-old_speed)*mps_variation;
	}
	if(fabs(turn-old_turn)>rps_variation){
	    turn=old_turn+(turn-old_turn)/fabs(turn-old_turn)*rps_variation;
	}
    }
    
    old_t= timer_ms();
    old_speed=speed;
    old_turn=turn;
    m_Velocity.v=speed;
    m_Velocity.w=turn;
    m_VelocityOut.write();
    return RTC::RTC_OK;
}


/*
RTC::ReturnCode_t RunControl::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RunControl::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RunControl::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RunControl::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RunControl::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void RunControlInit(RTC::Manager* manager)
  {
    RTC::Properties profile(runcontrol_spec);
    manager->registerFactory(profile,
                             RTC::Create<RunControl>,
                             RTC::Delete<RunControl>);
  }
  
};


